@@ -6,10 +6,9 @@
 
 #include "purpose/configuration.h"
 #include "externalprocess/processjob.h"
-#include <KPluginFactory>
 #include <QFileInfo>
 
-#include <KPluginLoader>
+#include <KPluginFactory>
 #include <KPluginMetaData>
 
 #include <QDebug>
@@ -32,12 +31,15 @@ public:
 
     static void checkJobFinish(KJob *job)
     {
-        QStringList outputArgs = job->property("outputArgs").toStringList();
-        QJsonObject output = job->property("output").toJsonObject();
+        const QStringList outputArgs = job->property("outputArgs").toStringList();
+        const auto argsSet = QSet<QString>{outputArgs.cbegin(), outputArgs.cend()};
 
-        if (!output.keys().toSet().contains(outputArgs.toSet()) && job->error() == 0) {
+        const QStringList outputKeys = job->property("output").toJsonObject().keys();
+        const auto keysSet = QSet<QString>{outputKeys.cbegin(), outputKeys.cend()};
+
+        if (!keysSet.contains(argsSet) && job->error() == 0) {
             qWarning() << "missing output values for" << job->metaObject()->className() << ". Expected: " << outputArgs.join(QStringLiteral(", "))
-                       << ". Got: " << output.keys().join(QStringLiteral(", "));
+                       << ". Got: " << outputKeys.join(QStringLiteral(", "));
         }
     }
 
@@ -56,19 +58,14 @@ public:
         if (fileName.endsWith(QLatin1String("/metadata.json"))) {
             return new ProcessJob(m_pluginData.fileName(), m_pluginTypeName, m_inputData, parent);
         } else {
-            KPluginLoader loader(fileName);
-            KPluginFactory *factory = loader.factory();
-            if (!factory) {
-                qWarning() << "Couldn't create job:" << fileName << loader.errorString();
-                return nullptr;
-            }
-            Purpose::PluginBase *plugin = dynamic_cast<Purpose::PluginBase *>(factory->create<QObject>(parent, QVariantList()));
+            auto pluginResult = KPluginFactory::instantiatePlugin<QObject>(m_pluginData, parent, QVariantList());
 
-            if (!plugin) {
-                qWarning() << "Couldn't load plugin:" << fileName << loader.errorString();
+            if (!pluginResult) {
+                qWarning() << "Couldn't load plugin:" << fileName << pluginResult.errorString;
                 return nullptr;
             }
 
+            Purpose::PluginBase *plugin = dynamic_cast<Purpose::PluginBase *>(pluginResult.plugin);
             return plugin->createJob();
         }
     }
@@ -150,7 +147,6 @@ Purpose::Job *Configuration::createJob()
     job->setProperty("outputArgs", d->m_pluginType.value(QStringLiteral("X-Purpose-OutboundArguments")));
 
     connect(job, &Purpose::Job::finished, &ConfigurationPrivate::checkJobFinish);
-    connect(job, &QObject::destroyed, this, &QObject::deleteLater);
     return job;
 }
 
